<?php
namespace app\controllers;

use app\models\addresses;
use app\models\users;
use app\models\emails;
use app\models\user_addresses;
use app\models\magic_links;

class account
{
    public static function index($defaults): void
    {
        if (!isset($_SESSION['user_id'])) {
            header('Location: /account/login');
        }
        $user_id = $_SESSION['user_id'];
        $user = users::getById($user_id);
        $default_shipping = null; 
        $default_billing = null; 
        $ship_addrs = [];
        $bill_addrs = [];
        $addresses = user_addresses::getShippingByUserId($user['id']);
        foreach ($addresses as $address) {
            if ($address['id'] == $user['shipping_address_id']){
                $default_shipping = $address;
            } else {
                $ship_addrs[] = $address;
            }
        }
        $bill_addresses = user_addresses::getBillingByUserId($_SESSION['user_id']);
        foreach ($bill_addresses as $addr) {
            if ($addr['id'] == $user['billing_address_id']){
                $default_billing = $addr;
            } else {
                $bill_addrs[] = $addr; 
            }
        }
        echo $GLOBALS['twig']->render('lib/page/index.twig', array_merge($defaults, [
            'child_template' => 'account/index.twig',
            'page_title' => 'Manage Account - ' . $_ENV['APP_NAME'],
            'user' => $user,
            'shipping' => $ship_addrs,
            'billing' => $bill_addrs,
            'default_shipping' => $default_shipping,
            'default_billing' => $default_billing,
            'breadcrumbs' => [
                [
                    'url' => null,
                    'title' => 'My Account',
                ]
            ]
        ]));
    }
    public static function billing($defaults)
    {
        if ($_SERVER['REQUEST_METHOD'] == 'POST') {
            if (!$_SESSION['user_id']) {
                http_response_code(403);
            }
            $bill = addresses::validatePost("billing");
            $bill_id = addresses::add(
                $bill['name'],
                $bill['company'],
                $bill['addressLine1'],
                $bill['addressLine2'],
                $bill['city'],
                $bill['state'],
                $bill['zip'],
                $bill['phone'],
                1,
                0
            );
            user_addresses::add(
                $_SESSION['user_id'],
                $bill_id
            );
            $_SESSION['success'] = "Billing address saved!";
            header('Location: /account/billing');
        }
        $user_id = $_SESSION['user_id'];
        $user = users::getById($user_id);
        $default_billing = null; 
        $bill_addrs = [];
        $bill_addresses = user_addresses::getBillingByUserId($_SESSION['user_id']);
        foreach ($bill_addresses as $addr) {
            if ($addr['id'] == $user['billing_address_id']){
                $default_billing = $addr;
            } else {
                $bill_addrs[] = $addr; 
            }
        }
        echo $GLOBALS['twig']->render('lib/page/index.twig', array_merge($defaults, [
            'child_template' => 'account/billing.twig',
            'page_title' => 'Billing Information - ' . $_ENV['APP_NAME'],
            'billing' => $bill_addrs,
            'default_billing' => $default_billing,
            'breadcrumbs' => [
                [
                    'url' => '/account',
                    'title' => 'My Account'
                ],
                [
                    'url' => null,
                    'title' => 'Billing'
                ]
            ]
        ]));
    }
    public static function profile()
    {
        if ($_SERVER['REQUEST_METHOD'] == 'POST') {
            if (!$_SESSION['user_id']) {
                http_response_code(403);
            }
            users::updateProfileById($_SESSION['user_id'], $_POST);
            header('Location: /account');
        }
    }

    public static function email()
    {
        $user_id = $_SESSION['user_id'] ?? null;
        if (empty($user_id)){
            header('Location: /account/login');
        }
        if ($_SERVER['REQUEST_METHOD'] == 'POST') {  
            $email = $_POST['email'] ?? null;
            if (empty($email)) {
                $_SESSION['error'] = "Enter your email to get a login link";
                header('Location: /account');
                exit;
            } else {
                $token = magic_links::add($email, $user_id);
                users::updateReplaceEmailTokenById($user_id, $token);
                header('Location: /account');
                exit;
            }
        }
    }

    public static function verify($defaults)
    {
        if ($_SERVER['REQUEST_METHOD'] == 'POST') {
            $code = $_POST['code'];
            $link = magic_links::validateCode($code);
            if ($link) {
                $user = $link['user_id'] ? users::getById($link['user_id']) : users::getByEmail($link['email']);
                if ($user) {
                    $_SESSION['user_email'] = $link['email'];
                    $_SESSION['user_id'] = $user['id'];
                    if (!$user['verified']) {
                        users::verify($link['email']);
                    }
                    header('Location: /account');
                    exit;
                } else {
                    $_SESSION['user_email'] = $link['email'];
                    header('Location: /account/signup');
                    exit;
                }
            } else {
                $_SESSION['error'] = "Invalid or expired verification code.";
                header('Location: /account/verify');
                exit;
            }
        }

        echo $GLOBALS['twig']->render('lib/page/index.twig', array_merge($defaults, [
            'child_template' => 'account/verify.twig',
            'page_title' => $_ENV['APP_NAME'],
            'breadcrumbs' => [
                [
                    'url' => '/account',
                    'title' => 'My Account'
                ],
                [
                    'url' => null,
                    'title' => 'Verify'
                ]
            ]
        ]));
    }

    public static function login($defaults)
    {
        if ($_SERVER['REQUEST_METHOD'] == 'POST') {
            $email = $_POST['email'] ?? false;
            if (empty($email)) {
                $_SESSION['error'] = "Enter your email to get a login link";
                header('Location: /account/login');
                exit;
            } else {
                $token = magic_links::add($email, null);
                header('Location: /account/verify');
                exit;
            }
        }
        if (isset($_SESSION['user_id'])) {
            header('Location: /account');
        }
        echo $GLOBALS['twig']->render('lib/page/index.twig', array_merge($defaults, [
            'child_template' => 'account/login.twig',
            'page_title' => 'Sign In or Create an Account!',
            'breadcrumbs' => [
                [
                    'url' => null,
                    'title' => 'My Account'
                ],
            ]
        ]));
    }

    public static function logout()
    {
        session_unset();
        session_destroy();
        header('Location: /');
    }

    public static function orders($defaults)
    {
        if (!isset($_SESSION['user_id'])) {
            header('Location: /account/login');
        }
        echo $GLOBALS['twig']->render('lib/page/index.twig', array_merge($defaults, [
            'child_template' => 'account/orders.twig',
            'page_title' => 'View ' . $_ENV['APP_NAME'] . ' Orders',
            'breadcrumbs' => [
                [
                    'url' => '/account',
                    'title' => 'My Account'
                ],
                [
                    'url' => null,
                    'title' => 'Orders'
                ]
            ]
        ]));
    }

    public static function returns($defaults)
    {
        echo $GLOBALS['twig']->render('lib/page/index.twig', array_merge($defaults, [
            'child_template' => 'account/returns.twig',
            'page_title' => 'View ' . $_ENV['APP_NAME'] . ' Returns',
            'breadcrumbs' => [
                [
                    'url' => '/account',
                    'title' => 'My Account'
                ],
                [
                    'url' => null,
                    'title' => 'Returns'
                ]
            ]
        ]));
    }
    public static function shipping($defaults)
    {
        if ($_SERVER['REQUEST_METHOD'] == 'POST') {
            if (!$_SESSION['user_id']) {
                http_response_code(403);
            }
            $ship = addresses::validatePost("shipping");
            $ship_id = addresses::add(
                $ship['name'],
                $ship['company'],
                $ship['addressLine1'],
                $ship['addressLine2'],
                $ship['city'],
                $ship['state'],
                $ship['zip'],
                $ship['phone'],
                0,
                1
            );
            user_addresses::add(
                $_SESSION['user_id'],
                $ship_id
            );
            $_SESSION['success'] = "Shipping address saved!";
            header('Location: /account/shipping');
        }
        $user_id = $_SESSION['user_id'];
        $user = users::getById($user_id);
        $addresses = user_addresses::getShippingByUserId($user['id']);
        $default_shipping = null; 
        $ship_addrs = [];
        foreach ($addresses as $addr) {
            if ($addr['id'] == $user['shipping_address_id']){
                $default_shipping = $addr;
            } else {
                $ship_addrs[] = $addr; 
            }
        }
        echo $GLOBALS['twig']->render('lib/page/index.twig', array_merge($defaults, [
            'child_template' => 'account/shipping.twig',
            'page_title' => $_ENV['APP_NAME'] . ' Shipping',
            'shipping' => $ship_addrs,
            'default_shipping' => $default_shipping,
            'breadcrumbs' => [
                [
                    'url' => '/account',
                    'title' => 'My Account'
                ],
                [
                    'url' => null,
                    'title' => 'Shipping'
                ]
            ]
        ]));
    }

    public static function signup($defaults)
    {
        if ($_SERVER['REQUEST_METHOD'] == 'POST') {
            $email = $_POST['email'];
            if (empty($email)) {
                $_SESSION['error'] = 'Email is required.';
            }
            $existingUser = users::getByEmail($email);
            if ($existingUser) {
                $_SESSION['error'] = 'Email already exists. Please choose a different email or log in.';
                $_SESSION['last_post'] = $_POST;
                header('Location: /account/signup');
                exit;
            }
            $useShipping = $_POST['use_shipping'] ?? false;
            $ship = addresses::validatePost("shipping");
            if (!isset($ship['name'])){
                $_SESSION['error'] = "Shipping address verification failed. Check your entry for errors.";
                $_SESSION['last_post'] = $_POST;
                header('Location: /account/signup');
            }
            if (!$useShipping) {
                $bill = addresses::validatePost("billing");
                if (!isset($bill['name'])){
                    $_SESSION['error'] = "Billing address verification failed. Check your entry for errors.";
                    $_SESSION['last_post'] = $_POST;
                    header('Location: /account/signup');
                }
            }
            if (isset($_SESSION['error'])) {
                $_SESSION['last_post'] = $_POST;
                header('Location: /account/signup');
            }
            $ship_id = addresses::add(
                $ship['name'],
                $ship['company'],
                $ship['addressLine1'],
                $ship['addressLine2'],
                $ship['city'],
                $ship['state'],
                $ship['zip'],
                $ship['phone'],
                $useShipping == 'on',
                1
            );
            $bill_id = $ship_id;
            if (!$useShipping) {
                $bill_id = addresses::add(
                    $bill['name'],
                    $bill['company'],
                    $bill['addressLine1'],
                    $bill['addressLine2'],
                    $bill['city'],
                    $bill['state'],
                    $bill['zip'],
                    $bill['phone'],
                    1,
                    0
                );
            }
            $opt_in_promotional = $_POST['opt_in_promotional'] ?? false;
            $verified = isset($_SESSION['user_email']);
            $dark_theme = $defaults['theme'] == 'dark';
            $user_id = users::add(
                $email,
                $ship_id,
                $bill_id,
                $opt_in_promotional,
                $verified,
                $dark_theme
            );
            emails::updateUserIdByEmail($email, $user_id);
            user_addresses::add(
                user_id: $user_id,
                address_id: $ship_id
            );
            if (!$useShipping) {
                user_addresses::add(
                    user_id: $user_id,
                    address_id: $bill_id
                );
            }
            $_SESSION['user_id'] = $user_id;
            if (!$verified) {
                header("Location: /magic-link?email=$email&signup=1");
                exit;
            } 
            header('Location: /account');
            exit;
        } // endif request === POST

        if (isset($_SESSION['user_id'])) {
            header('Location: /account');
            exit;
        }

        echo $GLOBALS['twig']->render('lib/page/index.twig', array_merge($defaults, [
            'child_template' => 'account/signup.twig',
            'page_title' => 'Create an Account - ' . $_ENV['APP_NAME']
        ]));
    }
}