<?php
namespace app\models;

use app\app;
class addresses
{
    public static function init()
    {
        app::$db->exec("CREATE TABLE IF NOT EXISTS addresses (
            id INTEGER PRIMARY KEY AUTOINCREMENT,
            name TEXT NOT NULL,
            company TEXT,
            addressLine1 TEXT NOT NULL,
            addressLine2 TEXT NOT NULL,
            city TEXT NOT NULL,
            state TEXT NOT NULL,
            zip TEXT NOT NULL,
            phone TEXT,
            billing BOOLEAN NOT NULL,
            shipping BOOLEAN NOT NULL
        )");
    }

    public static function validatePost($type)
    {
        $name = $_POST["{$type}_name"];
        $company = $_POST["{$type}_company"] ?? null;
        $addressLine2 = $_POST["{$type}_addressLine2"] ?? null;
        $addressLine1 = $_POST["{$type}_addressLine1"];
        $city = $_POST["{$type}_city"];
        $state = $_POST["{$type}_state"];
        $zip = $_POST["{$type}_zip"];
        $phone = $_POST["{$type}_phone"];
        // check all required fields are set
        if (empty($name) || empty($addressLine1) || empty($city) || empty($state) || empty($zip) || empty($phone)) {
            $_SESSION['error'] = "Missing required {$type} information.";
        }
        $url = "https://nominatim.openstreetmap.org/search?" . http_build_query([
            "q" => implode(" ", array_filter([$addressLine1, $addressLine2, $city, $state, $zip])),
            "format" => "json",
            "addressdetails" => 1,
            "limit" => 1
        ]);
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_USERAGENT, "AddressValidator/1.0");
        $response = curl_exec($ch);
        curl_close($ch);
        $data = json_decode($response, true);
        if (empty($data)) {
            return ["error" => "Address not found"];
        }
        $addressDetails = $data[0]['address'];
        return [
            'name'         => $name,
            'company'      => $company,
            "addressLine1" => ($addressDetails['house_number'] ?? '') . ' ' . ($addressDetails['building'] ?? '') . ' ' . ($addressDetails['road'] ?? ''),
            "addressLine2" => $addressLine2,
            "city"         => $addressDetails['city'] ?? $addressDetails['town'] ?? $addressDetails['village'] ?? '',
            "state"        => $addressDetails['state_code'] ?? ($addressDetails['state'] ?? ''),
            "zip"          => $addressDetails['postcode'] ?? '',
            'phone'        => $phone
        ];
    }

    public static function add($name, $company, $addressLine1, $addressLine2, $city, $state, $zip, $phone, $billing, $shipping)
    {
        $query = "INSERT INTO addresses (
            name, 
            company, 
            addressLine1,
            addressLine2, 
            city, 
            state, 
            zip,
            phone,
            billing,
            shipping
        ) VALUES (
            :name, 
            :company, 
            :addressLine1,
            :addressLine2,
            :city, 
            :state, 
            :zip,
            :phone,
            :billing,
            :shipping
        )";
        $stmt = app::$db->prepare($query);
        $stmt->bindParam(':name', $name);
        $stmt->bindParam(':company', $company);
        $stmt->bindParam(':addressLine1', $addressLine1);
        $stmt->bindParam(':addressLine2', $addressLine2);
        $stmt->bindParam(':city', $city);
        $stmt->bindParam(':state', $state);
        $stmt->bindParam(':zip', $zip);
        $stmt->bindParam(':phone', $phone);
        $stmt->bindParam(':billing', $billing);
        $stmt->bindParam(':shipping', $shipping);
        $stmt->execute();
        return app::$db->lastInsertId();
    }

}
